# -*- coding: utf-8 -*-
import sys
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk

#
#
class KeywordSelectDialog:
    '''
    キーワード選択用ダイアログ。
    '''
    def __init__(self, title='Choose keyword', parent=None, keywords=[]):
        '''
        初期化
        '''        
        self.keywords = keywords[:]
        
        self.dlog = gtk.Dialog(title,
                               parent,
                               gtk.DIALOG_MODAL,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_OK, gtk.RESPONSE_OK))
        self.dlog.set_border_width(5)
        self.dlog.set_resizable(False)
        self.dlog.set_position(gtk.WIN_POS_CENTER)

        contents = self.make_contents()
        if contents:
            self.dlog.vbox.pack_start(contents, False, False, 5)

        self.dlog.show_all()

    def make_contents(self):
        '''
        ダイアログの内容を構築する
        '''
        hbox = gtk.HBox()        
        label = gtk.Label('Keywords : ')
        hbox.pack_start(label)

        # コンボボックスの構築
        self.comboEntry = gtk.combo_box_entry_new_text()
        hbox.pack_start(self.comboEntry)
        for keys in self.keywords:
            if len(keys) < 2:
                continue
            text = keys[0] + ' (' + reduce(lambda a, b: a+', '+b, keys[1:]) + ')'
            self.comboEntry.append_text(text)

        return hbox

    def on_button_add(self, widget, event=None, data=None):
        '''
        キーワードを追加する。
        キーワードがまだ存在しない場合にのみ追加。
        '''
        text = self.get_active_keyword()
        self.append_keyword(text)

        return True
        
    def run(self):
        '''
        ダイアログのイベントループを開始
        OKボタンが押された時、コンボボックスに入力された文字列が
        新規のテキストの場合には、キーワードリストに追加する。
        '''
        response = self.dlog.run()
        if response == gtk.RESPONSE_OK:
            text = self.get_active_keyword()
            if text not in self.keywords:
                self.keywords.append(text)

        return response

    def destroy(self):
        '''
        ダイアログの破棄
        '''
        return self.dlog.destroy()

    def get_active_keyword(self):
        '''
        現在選択されているテキストを取得する。
        '''
        if self.keywords == None or len(self.keywords) == 0:
            return None
        
        idx = self.comboEntry.get_active()
        if idx < 0:
            return None

        return self.keywords[idx][1:]

    def get_keywords(self):
        '''
        保持しているキーワードのリストを取得する。
        '''
        return self.keywords
    
    def append_keyword(self, text):
        '''        
        渡されたテキストをコンボボックスに追加する。
        また、自身が管理しているキーワードリストへも追加する
        渡されたテキストが既にキーワードリスト中に存在
        する場合には何も行わない。
        '''
        if text == None or len(text) == 0:
            return False

        if text not in self.keywords:
            self.keywords.append(text)
            self.comboEntry.append_text(text)

        return True
    
#
#
def test():
    '''
    テスト
    '''
    textList = ['aaaa',
                'bbbb',
                'ccccccc']
    dlog = KeywordSelectDialog(keywords=textList)
    response = dlog.run()
    print dlog.get_active_keyword()
    print dlog.get_keywords()
    dlog.destroy()
    
#
#
if __name__ == '__main__':
    test()
