# -*- coding: utf-8 -*-

import sys
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk

from dialogs.fileSelectDialog import FileSelectDialog

# -*- coding: utf-8 -*-
import sys
import os.path

import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gobject

#
#
class FilterListModel:
    '''
    ファイルフィルタリスト用のデータモデルクラス
    '''
    def __init__(self, filters=[]):
        '''
        初期化
        '''
        self.store = gtk.ListStore(gobject.TYPE_STRING,
                                   gobject.TYPE_STRING)
        # フィルタの初期値を設定
        self.set_filters(filters)

    def get_model(self):
        '''
        リストモデルを取得する。
        '''
        return self.store
    
    def set_filters(self, filters):
        '''
        フィルタを設定する。
        現在登録されているフィルタを一旦クリアし、
        渡されたフィルタを追加する。
        '''
        if filters == None:
            return
        self.clear_filters()
        self.append_filters(filters)

    def get_filters(self):
        '''
        モデルに登録されているフィルタを取得する
        '''
        aIter = self.store.get_iter_first()
        if aIter == None:
            return []

        filterList = []
        while aIter != None:
            # フィルタ１行分をタプルにしてfileListに追加。
            # タプルは(フィルタ名, フィルタ1, フィルタ2...)
            filt = []
            name = self.store.get_value(aIter, 0)
            filt.append(name)
            
            filterStr = self.store.get_value(aIter, 1)
            # フィルタ文字中のスペースを削除し、カンマで分割したものを
            # フィルタとして追加する。
            filt += filterStr.replace(' ', '').split(',')
            filterList.append(tuple(filt))
            
            aIter = self.store.iter_next(aIter)

        return filterList

    def append_filters(self, filters):
        '''
        渡されたフィルタを登録する。
        '''
        if filters == None:
            return

        for f in filters:
            if len(f) > 1:
                filterStr = reduce(lambda a, b: a+', '+b, f[1:])
                self.store.append((f[0], filterStr))

    def remove_filters(self, pathList):
        '''
        iterListで指定されたRowを削除
        '''
        if pathList == None:
            return

        for aIter in [self.store.get_iter(path) for path in pathList]:
            self.store.remove(aIter)

    def clear_filters(self):
        '''
        登録されているフィルタ項目を全てクリアする。
        '''
        self.store.clear()

#
#
class FilterDispModel:
    '''
    フィルタ選択用リストの表示モデル
    '''
    def construct_view(self, model):
        '''
        ツリービューを生成する。
        '''
        # ツリービューの生成
        tree = gtk.TreeView(model)
        tree.set_rules_hint(True)

        sel = tree.get_selection()
        if sel:
            sel.set_mode(gtk.SELECTION_MULTIPLE)
        
        cellConfs = [('Filter Name', self.on_name_edited),
                     ('Filter Strings', self.on_filter_str_edited)]

        # フィルタのカラムを作成
        for i, confs in enumerate(cellConfs):
            cName, func = confs
            render = gtk.CellRendererText()
            render.set_property('editable', True)
            render.connect('edited', func, model)

            col = gtk.TreeViewColumn(cName, render, text=i)
            col.set_resizable(True)
            tree.append_column(col)
            
        return tree
    
    def on_name_edited(self, render, path, newText, model):
        '''
        フィルタ名称が修正された場合に実行する処理
        '''
        if model == None:
            return
        model[path][0] = newText

    def on_filter_str_edited(self, render, path, newText, model):
        '''
        フィルタが修正された場合に実行する処理
        '''
        if model == None:
            return
        model[path][1] = newText

#
#
class FCPPreferenceDialog:
    '''
    ツール設定用ダイアログ
    '''
    def __init__(self, parent=None, fileFilters=[], treeFilters=[]):
        '''
        初期化
        '''
        # ダイアログを生成
        self.current_folder = '~/'
        
        self.dlog = gtk.Dialog('Preference',
                               parent,
                               gtk.DIALOG_MODAL,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_OK, gtk.RESPONSE_OK))
        self.dlog.set_border_width(10)
        self.dlog.set_size_request(400, 300)
        self.dlog.set_position(gtk.WIN_POS_CENTER)

        self.notebook = gtk.Notebook()
        self.dlog.vbox.pack_start(self.notebook, True, True, 4)

        # 全般設定タブとその内容を作成
        widget = self.construct_general_widgets()
        if widget:
            self.notebook.append_page(widget, gtk.Label('General'))

        # ファイル選択ダイアログ用フィルタ設定タブと
        # その内容を作成
        widget, self.fileFilterModel = self.construct_filter_widgets(fileFilters)
        if widget:
            self.notebook.append_page(widget, gtk.Label('File Filters'))

        # リスト用フィルタ設定タブとその内容を作成
        widget, self.listFilterModel = self.construct_filter_widgets(treeFilters)
        if widget:
            self.notebook.append_page(widget, gtk.Label('Tree Filters'))

        # ダイアログの表示
        self.dlog.show_all()

    def construct_general_widgets(self):
        '''
        Generalタブ用のウィジェットを生成する。
        '''
        vbox = gtk.VBox()
        vbox.set_border_width(5)
        
        # デフォルトディレクトリ設定項目
        frame = gtk.Frame('Defualt Directory for the File dialog.')
        frame.set_size_request(-1, 40)

        hbox = gtk.HBox()
        hbox.set_border_width(2)
        self.defDirLabel = gtk.Label('~/')
        self.defDirLabel.set_alignment(0.04, 0.5)
        hbox.pack_start(self.defDirLabel)

        bt = gtk.Button('...')
        bt.set_size_request(30, -1)
        bt.connect('clicked', self.on_choose_default_dir)
        hbox.pack_start(bt, False, False, 4)
        
        frame.add(hbox)
        vbox.pack_start(frame, False, False, 6)
        
        # ログディレクトリ設定項目
        frame = gtk.Frame('Directory for the logfile.')
        frame.set_size_request(-1, 40)
        
        hbox = gtk.HBox()
        hbox.set_border_width(2)
        self.logDirLabel = gtk.Label('~/')
        self.logDirLabel.set_alignment(0.04,  0.5)
        hbox.pack_start(self.logDirLabel)

        bt = gtk.Button('...')
        bt.set_size_request(30, -1)
        bt.connect('clicked', self.on_choose_log_dir)
        hbox.pack_start(bt, False, False, 4)
        frame.add(hbox)
        vbox.pack_start(frame, False, False, 6)

        # コピー時の日付に関するオプション
        frame = gtk.Frame('Copy option')
        frame.set_size_request(-1, 40)

        hbox = gtk.HBox()
        hbox.set_border_width(2)

        self.checkKeepSrcDate = gtk.CheckButton('Keep the date of the source')
        hbox.pack_start(self.checkKeepSrcDate)
        frame.add(hbox)
        vbox.pack_start(frame, False, False, 6)
        
        return vbox

    def construct_filter_widgets(self, filters):
        '''
        フィルタ用ウィジェット（ツリービュー）
        を生成する。
        '''
        if filters == None:
            filters = []

        # データモデルを生成
        listModel = FilterListModel(filters)

        # 表示モデルを生成
        dispModel = FilterDispModel()

        tree = dispModel.construct_view(listModel.get_model())
        if tree == None:
            return None, None

        scroll = gtk.ScrolledWindow()
        scroll.add(tree)
        
        vbox = gtk.VBox()
        vbox.pack_start(scroll, True, True, 2)

        hbox = gtk.HBox()

        bt = gtk.Button('Clear')
        bt.connect('clicked', self.on_clear_filter, tree, listModel)
        hbox.pack_start(bt, True, True, 4)
        
        bt = gtk.Button('Remove')
        bt.connect('clicked', self.on_remove_filter, tree, listModel)
        hbox.pack_start(bt, True, True, 4)

        bt = gtk.Button('Add')
        bt.connect('clicked', self.on_add_filter, tree, listModel)
        hbox.pack_start(bt, True, True, 4)

        vbox.pack_start(hbox, False, False, 4)
        return vbox, listModel

    def run(self):
        '''
        ダイアログのイベント処理
        '''
        return self.dlog.run()

    def destroy(self):
        '''
        ダイアログの破棄
        '''
        self.dlog.destroy()

    def set_current_folder(self, path):
        '''
        プリファレンスダイアログで表示されるファイル選択ダイアログ
        で表示すべきディレクトリを設定する。
        '''
        self.current_folder = path

    def get_current_folder(self):
        '''
        プリファレンスダイアログで表示されるファイル選択ダイアログ
        で表示すべきディレクトリを設定する。
        '''
        return self.current_folder

    def get_default_dir(self):
        '''
        ファイルダイアログの初期ディレクトリパスを取得
        '''        
        return self.defDirLabel.get_text()

    def set_default_dir(self, path):
        '''
        ファイルダイアログの初期ディレクトリパスを取得
        '''
        self.defDirLabel.set_text(path)

    def get_log_dir(self):
        '''
        ログファイルの初期ディレクトリパスを取得
        '''
        return self.logDirLabel.get_text()
    
    def set_log_dir(self, path):
        '''
        ログファイルの初期ディレクトリパスを取得
        '''
        self.logDirLabel.set_text(path)

    def get_file_filters(self):
        '''
        ファイルフィルタを取得する。
        '''
        return self.fileFilterModel.get_filters()

    def get_list_filters(self):
        '''
        リストフィルタを取得する
        '''
        return self.listFilterModel.get_filters()

    def set_keep_file_date(self, flag):
        '''
        コピー時に、コピー元のファイル日付を維持するかどうかのフラグを設定
        '''
        self.checkKeepSrcDate.set_active(flag)

    def get_keep_file_date(self):
        '''
        コピー時に、コピー元のファイル日付を維持するかどうかのフラグを設定
        '''
        return self.checkKeepSrcDate.get_active()
        
    def on_choose_default_dir(self, widget, event=None, data=None):
        '''
        ディレクトリ選択ダイアログを表示
        '''
        dlg = FileSelectDialog(title='Choose directory for the file dialog default',
                               action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                               position=gtk.WIN_POS_CENTER)
        dlg.set_current_folder(self.current_folder)

        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            path = dlg.get_filenames()[0]
            self.defDirLabel.set_text(path)            

        self.current_folder = dlg.get_current_folder()
        dlg.destroy()
    
    def on_choose_log_dir(self, widget, event=None, data=None):
        '''
        ログ出力ディレクトリ選択ダイアログを表示
        '''
        dlg = FileSelectDialog(title='Choose directory for the log file',
                               action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                               position=gtk.WIN_POS_CENTER)
        dlg.set_current_folder(self.current_folder)

        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            path = dlg.get_filenames()[0]
            self.logDirLabel.set_text(path)
        
        self.current_folder = dlg.get_current_folder()
        dlg.destroy()

    def on_add_filter(self, widget, tree, filterModel):
        '''
        フィルタ追加ボタンが押された場合の処理
        新規のフィルタ行を追加。
        '''
        if filterModel == None:
            return True

        filterModel.append_filters([('New Filter', '*')])
        return True

    def on_remove_filter(self, widget, tree, filterModel):
        '''
        フィルタ削除ボタンが押された場合の処理
        選択されている行を削除。
        '''
        if tree == None:
            return True

        sel = tree.get_selection()
        if sel == None:
            return True

        model, pathList = sel.get_selected_rows()
        if model == None or pathList == None:
            return True

        filterModel.remove_filters(pathList)
        return True

    def on_clear_filter(self, widget, tree, filterModel):
        '''
        フィルタクリアボタンが押された場合の処理
        全てのフィルタを削除。
        '''
        if filterModel == None:
            return True

        filterModel.clear_filters()
        return True
