# -*- coding: utf-8 -*-
import sys
import os.path

import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gobject


#
#
class FCPModel:
    '''
    FCPツリー用のデータモデルクラス    
    '''
    def __init__(self, targets=[]):
        '''
        初期化
        '''
        self.store = gtk.TreeStore(gobject.TYPE_STRING,
                                   gobject.TYPE_STRING)
        self.set_targets(targets)

    def get_model(self):
        '''
        ツリーモデルを取得する
        '''
        return self.store

    def set_targets(self, targets):
        '''
        ターゲットを指定して、モデルを更新する。
        モデルにターゲットの情報を追加する前に、一旦全てをクリアする
        '''
        if targets == None:
            return

        self.clear_targets()
        self.append_targets(targets)

    def append_targets(self, srcList):
        '''
        処理元となるファイルリストに登録されているファイルパスを
        ツリーモデルに追加する。
        '''
        if srcList == None:
            return

        for srcPath, dstPath in srcList:
            # ディレクトリ名、ファイル名に分割
            srcPath = os.path.normpath(os.path.expanduser(srcPath))

            dname, fname = os.path.split(srcPath)
            if len(fname) > 0:
                # パスがファイルパスだったら、
                # まずモデル中で追加すべきディレクトリ用Rowのイテレータ
                # を探す。
                dname, fname = os.path.split(srcPath)
                aIter = self.find_iter_dir(self.store.get_iter_first(),
                                           dname)
                if aIter == None:
                    # ディレクトリ用Rowのイテレータが存在しない場合には
                    # トップレベルの階層に新規のRow追加して
                    # その子としてファイル名のRowを追加する。
                    parent = self.store.append(None, (dname, None))
                    self.store.append(parent, (fname, dstPath))
                    
                else:
                    # ディレクトリ用Rowのイテレータが存在した場合には、
                    # 既にその子供として同名のファイル名が登録されて
                    # いないか調べ、存在しない場合にのみファイル名
                    # の行を追加する。
                    childIter = self.store.iter_children(aIter)
                    while childIter != None:
                        if self.store.get_value(childIter, 0) == fname:
                            break
                        childIter = self.store.iter_next(childIter)
                    else:
                        # while文をブレークせずに通過、すなわち
                        # 同名のものが存在しなかった場合にのみ実行される。
                        self.store.append(aIter, (fname, dstPath))

    def find_iter_dir(self, aIter, dname):
        '''
        aIterで指定されたのと同じ階層のRowに関して
        dnameで指定された文字列をもつ行を探す。
        存在しない場合にはNoneを返す。
        '''
        if aIter == None:
            return None
        
        if self.store.get_value(aIter, 0) == dname:
            return aIter

        return self.find_iter_dir(self.store.iter_next(aIter),
                                  dname)

    def get_src_iters(self, srcList):
        '''
        srcList中で指定されたコピー元ファイルパス行のイテレータ
        のリストを取得する。
        '''
        if srcList == None or len(srcList) == 0:
            return None

        iterList = []
        
        aIter = self.store.get_iter_first()
        while aIter != None:
            childIter = self.store.iter_children(aIter)
            while childIter != None:
                # ツリーモデルからディレクトリ文字列、ファイル名文字列を
                # 取得し、パス文字列を作る
                dirName = self.store.get_value(aIter, 0)
                fileName = self.store.get_value(childIter, 0)
                filePath = os.path.join(dirName, fileName)
                
                # パス文字列が渡されたsrcList中に存在したら
                # ファイル名の行のイテレータをリターン値用の
                # リストに登録する。
                if filePath in srcList:
                    iterList.append(childIter)
                    
                # 次のファイル行のイテレータを取得
                childIter = self.store.iter_next(childIter)
            # 次のディレクトリ行のイテレータを取得
            aIter = self.store.iter_next(aIter)

        return iterList
        
    def set_destination(self, dstPath, rows):
        '''
        rowsで指定された行にdstPathで指定された文字列を
        設定する。
        クリアしたい場合にはdstPathにNoneを指定する。
        '''
        iterList = []
        # 設定対象のRowを抽出する
        for r in rows:
            aIter = self.store.get_iter(r)
            childIter = self.store.iter_children(aIter)
            if childIter:
                # 子供のRowがある場合にはそれらのイテレータを
                # 設定対象リストへ登録
                while childIter != None:
                    iterList.append(childIter)
                    childIter = self.store.iter_next(childIter)
            else:
                # 子供の行の場合、まだ設定対象リストへ登録されて
                # いなければ登録する。
                if aIter not in iterList:
                    iterList.append(aIter)

        # 抽出したRowに対して文字列を設定する
        for aIter in iterList:
            # まだ出力対象パスが指定されていない場合、処理元のファイル名
            # を使用するが、既に設定されている場合には出力先のパスから
            # ファイル名を切り出して使用する。
            dName = self.store.get_value(aIter, 0)
            curDstPath = self.store.get_value(aIter, 1)
            if curDstPath != None:
                dDir, curDName = os.path.split(curDstPath)
                if len(curDName):
                    dName = curDName

            # 渡された出力先ディレクトリパスとファイル名を連結して
            # パス文字列を作成し、モデルに設定する
            path = None
            if dstPath != None:
                path = os.path.join(dstPath, dName)
            self.store.set_value(aIter, 1, path)

    def set_indexed_destination(self, origin, format, keepExt):
        '''
        出力先ファイル名をインデックス化（通し番号）する
        出力先のディレクトリが設定されていない行が見付かった
        場合、その時点で処理を中断する。
        '''
        aIter = self.store.get_iter_first()
        index = origin
        
        while aIter != None:
            childIter = self.store.iter_children(aIter)
            while childIter != None:
                # 処理元のファイル名が指定されていない場合には
                # 処理を中断
                srcPath = self.store.get_value(childIter, 0)
                if srcPath == None or len(srcPath) == 0:
                    return
                srcFile, srcExt = os.path.splitext(os.path.split(srcPath)[1])
                
                # 出力ファイル名が設定されていない場合には
                # 処理を中断
                dstPath = self.store.get_value(childIter, 1)
                if dstPath == None or len(dstPath) == 0:
                    return
                dstDir, dstFile = os.path.split(dstPath)
                if dstDir == None or len(dstDir) == 0:
                    return

                # インデックス化したファイル名を作成。
                idxFName = format.replace('%d', str(index))

                # 処理元の拡張子を使用するオプションkeepExtがTrue
                # に指定されている場合、インデックス化したファイル名
                # の拡張子を調べ、付加する。
                if keepExt:
                    idxName, idxExt = os.path.splitext(idxFName)
                    if idxExt != srcExt:
                        idxFName += srcExt
                    
                # 出力先ディレクトリにインデックス化したファイル名
                # を連結してパス文字列を作成。
                path = os.path.join(dstDir, idxFName)
                
                # 出力ファイルパスとしてモデルに設定
                self.store.set_value(childIter, 1, path)
                index += 1                

                # 次のファイル名の行のイテレータを取得
                childIter = self.store.iter_next(childIter)

            aIter = self.store.iter_next(aIter)
    
    def get_targets(self):
        '''
        モデルに登録されているターゲット情報をリストとして
        取得する。
        '''
        targets = []

        # トップ階層のRowのイテレータを取得
        aIter = self.store.get_iter_first()
        while aIter != None:            
            # トップ階層の子供のRowのイテレータを取得
            childIter = self.store.iter_children(aIter)
            while childIter != None:
                # 処理元ファイルのパスを作成
                srcPath = os.path.join(self.store.get_value(aIter, 0),
                                       self.store.get_value(childIter, 0))
                # 出力先ファイルパスを取得
                dstPath = self.store.get_value(childIter, 1)

                # タプルにして戻り値用のリストへ追加
                targets.append((srcPath, dstPath))
            
                # 次の子供のイテレータを取得
                childIter = self.store.iter_next(childIter)
                
            aIter = self.store.iter_next(aIter)

        return targets

    def remove_targets(self, rows):
        '''
        選択中のRowを削除する。
        '''
        parentList = []
        iterList = []
        # 削除対象の行を抽出する
        for r in rows:
            aIter = self.store.get_iter(r)
            childIter = self.store.iter_children(aIter)
            if childIter:
                # 親Rowのイテレータを登録しておく
                parentList.append(aIter)
                
                # 子供のRowがある場合にはそれらのイテレータを
                # 削除対象リストへ登録
                while childIter != None:
                    iterList.append(childIter)
                    childIter = self.store.iter_next(childIter)
            else:
                # 子供の行の場合、まだ削除対象リストへ登録されて
                # いなければ登録する。
                if aIter not in iterList:
                    iterList.append(aIter)

        # 削除対象リストに登録されているRowを削除する
        for aIter in iterList:
            self.store.remove(aIter)

        # 子供を持たなくなってしまった親Rowを削除する    
        for aIter in parentList:
            if not self.store.iter_has_child(aIter):
                self.store.remove(aIter)
        
    def clear_targets(self):
        '''
        保持しているターゲットモデルの内容を全て削除する
        '''
        self.store.clear()

    def iter_files_by_extension(self, extensions, match=True):
        '''
        指定された拡張子を持つ処理元ファイルの行のイテレータ
        のリストを取得する。 
        '''
        iterList = []

        # 引数matchがTrueに設定されている場合には、
        # 一致しているかどうかを判定する関数を。
        # matchがFalseの場合には、不一致かどうかを
        # 判定する関数を作成。
        compFunc = None
        if match:
            compFunc = lambda e, exts: e in exts
        else:
            compFunc = lambda e, exts: e not in exts
            
        # トップレベルの行(ディレクトリ行)イテレータを取得し、
        # 全てのトップレベルの行の子供(ファイル名の行)の中
        # に指定された拡張子を持つファイルがあるかどうか
        # を調べる。
        aIter = self.store.get_iter_first()
        while aIter != None:
            childIter = self.store.iter_children(aIter)
            while childIter != None:
                filename = self.store.get_value(childIter, 0)
                name, ext = os.path.splitext(filename)

                # 判定関数がTrueを返す場合のみ、
                # イテレータを選択行としてリストに登録する。
                if compFunc(ext, extensions):
                    iterList.append(childIter)

                childIter = self.store.iter_next(childIter)
                
            aIter = self.store.iter_next(aIter)
            
        return iterList

    def replace_string(self, searchText, replaceText, targetStr):
        '''
        文字列の置き換え
        '''
        if searchText == None or len(searchText) == 0:
            return

        funcDict = {'SRC': self.replace_src_string,
                    'DST': self.replace_dst_string,
                    'BOTH': self.replace_both_string}

        if targetStr not in funcDict:
            return

        funcDict[targetStr](searchText, replaceText)

    def replace_src_string(self, searchText, replaceText):
        '''
        コピー元のファイルパス文字列(ファイル名は含まない)中のsearchText文字列を
        replaceTextに置き換える
        '''
        aIter = self.store.get_iter_first()
        while aIter != None:
            text = self.store.get_value(aIter, 0)
            text = text.replace(searchText, replaceText)
            self.store.set_value(aIter, 0, text)
            aIter = self.store.iter_next(aIter)
            
    def replace_dst_string(self, searchText, replaceText):
        '''
        コピー先のファイルパス文字列中のsearchText文字列を
        replaceTextに置き換える
        '''
        print searchText
        print replaceText
        
        aIter = self.store.get_iter_first()
        while aIter != None:
            childIter = self.store.iter_children(aIter)
            
            while childIter != None:
                text = self.store.get_value(childIter, 1)
                if text:
                    text = text.replace(searchText, replaceText)
                    self.store.set_value(childIter, 1, text)
                childIter = self.store.iter_next(childIter)

            aIter = self.store.iter_next(aIter)

    def replace_both_string(self, searchText, replaceText):
        '''
        全てのファイルパス文字列中のsearchText文字列を
        replaceTextに置き換える
        '''
        self.replace_src_string(searchText, replaceText)
        self.replace_dst_string(searchText, replaceText)

    
#
#
class FCPDispModel:
    '''
    FCPツリー用のデータモデル表示用クラス
    '''
    def construct_view(self, model, enableTreeLine=True):
        '''
        初期化
        '''
        # ツリービューの生成
        tree = gtk.TreeView(model)
        tree.set_rules_hint(True)
        tree.set_enable_tree_lines(True)
        tree.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_VERTICAL)

        # セルレンダラの生成
        cellConfs = [('Src', self.on_src_edited, False),
                     ('Dst', self.on_dst_edited, True)]
        
        for i, conf in enumerate(cellConfs):
            cName, func, editable = conf
            render = gtk.CellRendererText()
            render.set_property('editable', editable)
            render.connect('edited', func, model)

            col = gtk.TreeViewColumn(cName, render, text=i)
            col.set_resizable(True)
            tree.append_column(col)

        tree.connect('size_allocate', self.on_size_allocated)
        
        return tree

    def on_size_allocated(self, widget, allocation, data=None):
        '''
        TreeViewのサイズが決定した場合に実行すべき処理。
        TreeViewの全てのカラムのサイズを等分する。
        '''
        if allocation == None:
            return
        if not isinstance(widget, gtk.TreeView):
            return

        colList = widget.get_columns()
        colWidth = allocation.width / len(colList)

        for col in colList:
            col.set_min_width(colWidth)

    def on_src_edited(self, render, path, newText, model):
        '''
        Srcカラムが変更された場合の処理。
        モデルに変更を反映させる。
        '''
        if model == None:
            return
        model[path][0] = newText

    def on_dst_edited(self, render, path, newText, model):
        '''
        Dstカラムが変更された場合の処理。
        モデルに変更を反映させる。
        '''
        if model == None:
            return
        model[path][1] = newText
