# -*- coding: utf-8 -*-

import types
import  xml.sax
from xml.dom.minidom import Document
from xml.sax.handler import ContentHandler

PREF_KEY_SAVE_WINDW_POS = 'save_window_pos'
PREF_KEY_SAVE_WINDW_SIZE = 'save_window_size'
PREF_KEY_WINDOW_POS = 'window_pos'
PREF_KEY_WINDOW_SIZE = 'window_size'
PREF_KEY_DEFAULT_DIRECTORY = 'default_dir'
PREF_KEY_KEEP_FILE_DIRECTORY = 'keep_file_directory'
PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT = 'save_file_dir_as_default'
PREF_KEY_VIEW_TREE_EXPANDED = 'expanded_tree'
PREF_KEY_VIEW_SELECT_APPENDED_ROWS = 'select_appended_row'
PREF_KEY_VIEW_DRAW_TREE_LINE = 'draw_tree_line'
PREF_KEY_VIEW_SHOW_TOOLBAR = 'show_toolbar'
PREF_KEY_LIST_FILTER_EXTENSIONS = 'list_filter_extentions'
PREF_KEY_FILE_FILTER_EXTENSIONS = 'file_filter_extentions'
PREF_KEY_FILE_KEEP_SOURCE_DATE = 'keep_source_date'
PREF_KEY_LOG_EXPORT = 'log_export'
PREF_KEY_LOG_DIRECTORY = 'log_directory'
PREF_KEY_LOG_APPEND_MODE = 'log_append_mode'
PREF_KEY_CHILD = 'item'

ELEMENT_NAME_ROOT = 'fcp_prefs'
ELEMENT_ATTR_VALUE = 'val'
ELEMENT_ATTR_TYPE = 'type'
ELEMENT_ATTR_TYPE_STR_SEQUENCE = 'STR_SEQUENCE'
ELEMENT_ATTR_TYPE_INT_SEQUENCE = 'INT_SEQUENCE'
ELEMENT_ATTR_TYPE_BOOL_SEQUENCE = 'BOOL_SEQUENCE'


#
#
class FCPPrefGenerator:
    '''
    プリファレンスデータをXMLに変換するクラス
    '''
    def __init__(self):
        '''
        初期化
        '''
        # XML DOMドキュメントを生成し、
        # rootとなるタグを子供として追加しておく。
        self.doc = Document()
        self.root = self.doc.createElement(ELEMENT_NAME_ROOT)
        self.doc.appendChild(self.root)

        self.genFuncDict = {
            PREF_KEY_SAVE_WINDW_POS: self.gen_boolean,
            PREF_KEY_SAVE_WINDW_SIZE: self.gen_boolean,
            PREF_KEY_WINDOW_POS: self.gen_sequence,
            PREF_KEY_WINDOW_SIZE: self.gen_sequence,
            PREF_KEY_DEFAULT_DIRECTORY: self.gen_string,
            PREF_KEY_KEEP_FILE_DIRECTORY: self.gen_boolean,
            PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT: self.gen_boolean,
            PREF_KEY_VIEW_TREE_EXPANDED: self.gen_boolean,
            PREF_KEY_VIEW_SELECT_APPENDED_ROWS: self.gen_boolean,
            PREF_KEY_VIEW_DRAW_TREE_LINE: self.gen_boolean,
            PREF_KEY_VIEW_SHOW_TOOLBAR: self.gen_boolean,
            PREF_KEY_LIST_FILTER_EXTENSIONS: self.gen_some_str_sequences,
            PREF_KEY_FILE_FILTER_EXTENSIONS: self.gen_some_str_sequences,
            PREF_KEY_FILE_KEEP_SOURCE_DATE: self.gen_boolean,
            PREF_KEY_LOG_EXPORT: self.gen_boolean,
            PREF_KEY_LOG_DIRECTORY: self.gen_string,
            PREF_KEY_LOG_APPEND_MODE: self.gen_boolean}

    def __del__(self):
        '''
        破棄
        '''
        self.doc.unlink()

    def gen_boolean(self, tagStr, val):
        '''
        bool値を文字列に変換し、属性として追加したエレメント
        を生成する。
        '''
        if tagStr == None or len(tagStr) == 0:
            return None
        if val == None:
            return None
        elm = self.doc.createElement(tagStr)
        elm.setAttribute(ELEMENT_ATTR_VALUE, str(val))
        return elm
    
    def gen_sequence(self, tagStr, seq):
        '''
        値のシーケンスの要素をカンマ区切りの文字列に変換し、
        属性として追加したエレメントを生成する。
        '''
        if tagStr == None or len(tagStr) == 0:
            return None
        if seq == None or len(seq) == 0:
            return None

        elm = self.doc.createElement(tagStr)
        seqStr = reduce(lambda a, b: str(a)+','+str(b), seq)
        elm.setAttribute(ELEMENT_ATTR_VALUE, seqStr)
        return elm    

    def gen_string(self, tagStr, s):
        '''
        文字列を属性として追加したエレメントを生成する。
        '''
        if tagStr == None or len(tagStr) == 0:
            return None
        if s == None:
            return None

        elm = self.doc.createElement(tagStr)
        elm.setAttribute(ELEMENT_ATTR_VALUE, s)
        return elm

    def gen_some_str_sequences(self, tagStr, some_seq):
        '''
        シーケンスを要素に持つシーケンスを文字列に変換して
        属性として追加したエレメント(itemタグ)を生成し、
        それらを子供に持つエレメントを生成する。        
        '''
        if tagStr == None or len(tagStr) == 0:
            return None
        if some_seq == None or len(some_seq) == 0:
            return None

        elm = self.doc.createElement(tagStr)
        elm.setAttribute(ELEMENT_ATTR_TYPE, ELEMENT_ATTR_TYPE_STR_SEQUENCE)
        for seq in some_seq:
            childElm = self.gen_sequence(PREF_KEY_CHILD, seq)
            elm.appendChild(childElm)
        return elm
    
    def generate(self, prefDict, encode='utf-8'):
        '''
        XMLを生成する
        '''
        for key, val in prefDict.iteritems():
            # プリファレンス設定項目ごとのタグを作成
            if key not in self.genFuncDict:
                continue

            # DOMのエレメントを生成する
            func = self.genFuncDict[key]
            elm = func(key, val)
            if elm == None:
                continue
            
            self.root.appendChild(elm)

        # xmlテキストを生成
        return self.doc.toprettyxml(encoding=encode)
    
#
#
class FCPPrefHandler(ContentHandler):
    '''
    XMLからプリファレンスを復元するXMLパーサ用ハンドラ
    '''
    def __init__(self):
        '''
        初期化
        '''
        ContentHandler.__init__(self)

        # 展開結果を保持するための辞書
        self.prefDict = {}

        self.parentKey = None
        self.childType = None
        
        # 設定項目名(XMLタグ名)リスト
        # このリストに登録されていないタグは無視。        
        self.startFuncDict = {
            PREF_KEY_SAVE_WINDW_POS: self.conv_boolean,
            PREF_KEY_SAVE_WINDW_SIZE: self.conv_boolean,
            PREF_KEY_WINDOW_POS: self.conv_tuple,
            PREF_KEY_WINDOW_SIZE: self.conv_tuple,
            PREF_KEY_DEFAULT_DIRECTORY: self.conv_string,
            PREF_KEY_KEEP_FILE_DIRECTORY: self.conv_boolean,
            PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT:self.conv_boolean,
            PREF_KEY_VIEW_TREE_EXPANDED: self.conv_boolean,
            PREF_KEY_VIEW_SELECT_APPENDED_ROWS: self.conv_boolean,
            PREF_KEY_VIEW_DRAW_TREE_LINE: self.conv_boolean,
            PREF_KEY_VIEW_SHOW_TOOLBAR: self.conv_boolean,
            PREF_KEY_LIST_FILTER_EXTENSIONS: self.keep_parent_info,
            PREF_KEY_FILE_FILTER_EXTENSIONS: self.keep_parent_info,
            PREF_KEY_FILE_KEEP_SOURCE_DATE: self.conv_boolean,
            PREF_KEY_LOG_EXPORT: self.conv_boolean,
            PREF_KEY_LOG_DIRECTORY: self.conv_string,
            PREF_KEY_LOG_APPEND_MODE: self.conv_boolean,
            PREF_KEY_CHILD: self.conv_some_str_sequence}

        self.endFuncDict = {
            PREF_KEY_LIST_FILTER_EXTENSIONS: self.clear_parent_info,
            PREF_KEY_FILE_FILTER_EXTENSIONS: self.clear_parent_info}

    def conv_boolean(self, tag, attrs):
        '''
        文字列「True」/「False」をbool値のTrue, Falseに変換する。
        '''
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None

        boolDict = {'True': True, 'False': False}
        
        val = attrs.getValue(ELEMENT_ATTR_VALUE)
        if val not in boolDict:
            return None

        return boolDict[val]
        
    def conv_list(self, tag, attrs):
        '''
        カンマ区切りの文字列をシーケンスに変換する
        '''
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None
        s = attrs.getValue(ELEMENT_ATTR_VALUE)
        if s == None or len(s) == 0:
            return []

        # カンマで区切った文字列を整数値のリストに変換
        # 現状では整数のタプル以外は想定していない。
        return [int(token) for token in s.split(',')]

    def conv_tuple(self, tag, attrs):
        '''
        カンマ区切りの文字列をシーケンスに変換する
        '''
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None
        s = attrs.getValue(ELEMENT_ATTR_VALUE)
        if s == None or len(s) == 0:
            return ()

        # カンマで区切った文字列を整数値のリストに変換
        # 現状では整数のタプル以外は想定していない。
        return tuple([int(token) for token in s.split(',')])

    def conv_str_tuple(self, tag, attrs):
        '''
        カンマ区切りの文字列をシーケンスに変換する
        '''
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None
        s = attrs.getValue(ELEMENT_ATTR_VALUE)
        if s == None or len(s) == 0:
            return ()

        # カンマで区切った文字列を整数値のリストに変換
        # 現状では整数のタプル以外は想定していない。
        return tuple([token for token in s.split(',')])

    def conv_string(self, tag, attrs):
        '''
        文字列を取得する。（変換は不要）
        '''
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None
        return attrs.getValue(ELEMENT_ATTR_VALUE)

    def conv_some_str_sequence(self, tag, attrs):
        '''
        文字列のシーケンスを取得する
        '''
        if self.parentKey == None or self.childType == None:
            return
        if attrs.has_key(ELEMENT_ATTR_VALUE) == False:
            return None

        return self.conv_str_tuple(attrs.getValue(ELEMENT_ATTR_VALUE), attrs)
        
    def keep_parent_info(self, name, attrs):
        '''
        親のエレメントの情報を退避しておく
        '''
        self.parentKey = name

        if attrs.has_key(ELEMENT_ATTR_TYPE):
            self.childType = attrs.getValue(ELEMENT_ATTR_TYPE)
        self.prefDict[name] = []

    def clear_parent_info(self, name):
        '''
        親のエレメントの情報をクリアする
        '''
        self.parentKey = None
        self.childType = None

    def get_pref_dict(self):
        '''
        パース処理結果を取得
        '''
        return self.prefDict
    
    def startElement(self, name, attrs):
        '''
        開始タグが出現するたびにパーサから呼び出される関数
        '''
        if name not in self.startFuncDict:
            return

        val = self.startFuncDict[name](name, attrs)
        if val:
            if self.parentKey:
                self.prefDict[self.parentKey].append(val)
            else:
                self.prefDict[name] = val

    def endElement(self, name):
        '''
        終了タグが出現するたびにパーサから呼び出される関数
        '''
        if name not in self.endFuncDict:
            return
        self.endFuncDict[name](name)
        
#
#
class FCPPreference:
    '''
    pyFCPの動作オプションを保持するクラス。
    '''
    def __init__(self):
        '''
        初期化
        '''
        # プリファレンスデータは辞書として保持される。
        # 各設定項目は以下の形となっている。
        # {設定項目名: {値タイプキー: 値タイプ, 値キー: 値}}
        self.prefDict = {
            # ウインドウの座標を保存するかどうか
            PREF_KEY_SAVE_WINDW_POS: False,
            
            # ウインドウのサイズを保存するかどうか
            PREF_KEY_SAVE_WINDW_SIZE: True,
            
            # ウインドウ左肩座標
            PREF_KEY_WINDOW_POS: (100, 100),

            # ウインドウサイズ
            PREF_KEY_WINDOW_SIZE: (500, 400),

            # ファイル選択ダイアログのデフォルトディレクトリ
            PREF_KEY_DEFAULT_DIRECTORY: './',

            # ファイル選択ダイアログのディレクトリを保持
            PREF_KEY_KEEP_FILE_DIRECTORY: True,

            # ファイル選択ダイアログのディレクトリの
            # 情報をデフォルトディレクトリとして反映させる
            # (ツール終了時)
            PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT: False,
            
            # ツリーを常に展開状態にするかどうか
            PREF_KEY_VIEW_TREE_EXPANDED: True,

            # 追加された行を選択状態にするかどうか
            PREF_KEY_VIEW_SELECT_APPENDED_ROWS: True,
            
            # ツリーにラインを描画するかどうか
            PREF_KEY_VIEW_DRAW_TREE_LINE: True,

            # ツールバーを表示するかどうか
            PREF_KEY_VIEW_SHOW_TOOLBAR: False,

            # ファイル日付はコピー元ファイル名を使用するかどうか
            PREF_KEY_FILE_KEEP_SOURCE_DATE: True,
            
            # ログ出力するかどうか
            PREF_KEY_LOG_EXPORT: True,
            
            # ログ出力ディレクトリ
            PREF_KEY_LOG_DIRECTORY: './',

            # ログを上書きモードで出力するかどうか。
            PREF_KEY_LOG_APPEND_MODE: False,
            
            # ツリーフィルタ用拡張子リスト
            PREF_KEY_LIST_FILTER_EXTENSIONS: [('py source', '.py'),
                                              ('py compiled source', '.pyc'),
                                              ('c source' , '.c'),
                                              ('c++ source', '.cpp'),
                                              ('text file', '.txt'),
                                              ('document file', '.doc')],

            # ファイルフィルタ用拡張子リスト
            PREF_KEY_FILE_FILTER_EXTENSIONS: [('All files', '*'),
                                              ('Image files', '*.jpg',
                                               '*.png', '*.tif',
                                               '*.gif', '*.xpm'),
                                              ('Text files', '*.txt',
                                               '*.c', '*.cpp',
                                               '*.h', '*.py'),
                                              ('Backup files',
                                               '*.bak', '*.*~')]}
    
    def get_value(self, key):
        '''
        設定値の取得
        '''
        if key not in self.prefDict:
            return None
        return self.prefDict[key]

    def set_value(self, key, value):
        '''
        値の設定
        '''
        if key not in self.prefDict:
            print 'unknown key'
            return

        self.prefDict[key] = value

    def set_xml_text(self, text):
        '''
        XMLから設定値の読み込み
        '''
        parser = xml.sax.make_parser()            
        handler = FCPPrefHandler()
        parser.setContentHandler(handler)
        try:
            parser.feed(text)
        except KeyError, err:
            print 'xml.sax KeyError : %s' % err.message

        # パースした結果をそのままself.prefDictに代入
        # すると危険なので、現在のプリファレンスのキーに
        # 一致している項目のみ反映させるようにする。
        prefParsed = handler.get_pref_dict()
        for key, val in prefParsed.iteritems():
            if key in self.prefDict:
                self.prefDict[key] = val
        
    def get_xml_text(self):
        '''
        設定値をXMLテキストとして書き出す
        '''
        gen = FCPPrefGenerator()
        text = gen.generate(self.prefDict)
        return text

#
# 以下はテスト用コード

#
#
def test_fcp_preference():
    '''
    プリファレンス生成テスト
    '''
    pref = FCPPreference()

    keyList = [(PREF_KEY_WINDOW_POS, (10, 10)),
               (PREF_KEY_WINDOW_SIZE, (100, 100)),
               (PREF_KEY_DEFAULT_DIRECTORY, '../../../'),
               (PREF_KEY_KEEP_FILE_DIRECTORY, True),
               (PREF_KEY_VIEW_TREE_EXPANDED, False),
               (PREF_KEY_VIEW_DRAW_TREE_LINE, False),
               (PREF_KEY_VIEW_SHOW_TOOLBAR, True),
               (PREF_KEY_LIST_FILTER_EXTENSIONS, [('name aaa', '.aaa'),
                                                  ('name bbb', '.bbb')])]

    for k, v in keyList:
        print k, pref.get_value(k)

    for k, v in keyList:
        pref.set_value(k, v)

    for k, v in keyList:
        print k, pref.get_value(k)

#
#
def test_fcp_pref_xml():
    '''
    プリファレンスのデータをXMLへ変換するテスト
    '''
    pref = FCPPreference()
    gen = FCPPrefGenerator()

    xmlText = gen.generate(pref.prefDict)
    print xmlText

    return xmlText

#
#
def test_fcp_pref_handler(text):
    '''
    XMLからぷリファレンスデータを復元するテスト
    '''
    parser = xml.sax.make_parser()            
    handler = FCPPrefHandler()
    parser.setContentHandler(handler)

    try:
        parser.feed(text)
        prefDict = handler.get_pref_dict()
        for k, v in prefDict.iteritems():
            print k, v
    except KeyError, err:
        print 'xml.sax KeyError : %s' % err.message
                             
#
#
if __name__ == '__main__':    
    import xml.sax

#    test_fcp_preference()

    xmlText = test_fcp_pref_xml()

    test_fcp_pref_handler(xmlText)

