# -*- coding: utf-8 -*-

import threading
import time
import os
import sys

import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk

from statusView import StatusView

#
#
class FCPRunThread(threading.Thread):
    '''
    FCPのファイルコピースレッド
    '''
    def __init__(self, targets, statusView=None,
                 cbError=None, cbFinished=None):
        '''
        初期化
        '''
        threading.Thread.__init__(self)

        self.targets = [(s, d) for s, d in targets if d != None]
        self.statusView = statusView
        self.exit = False
        self.errorCnt = 0
        self.cbError = cbError
        self.cbFinished = cbFinished
        self.copyFunc = None

    def set_copy_func(self, func):
        '''
        コピーを実行する関数を設定
        '''
        self.copyFunc = func
        
    def run(self):
        '''
        処理の実行
        '''
        # コピー実行関数が未指定の場合には処理を中止
        if self.copyFunc == None:
            print 'copy function is None'
            return
        
        # 処理を開始する直前に実行するコールバックを
        # 呼び出す
        print 'start'
        
        # エラー数のクリア
        self.errorCnt = 0

        # ステータスバーの状態を初期化
        if self.statusView:
            self.statusView.set_status_text('')
            self.statusView.set_progress_range(len(self.targets))
            self.statusView.set_fraction(0)

        # 全てのターゲットに対して処理を実行する。
        for src, dst in self.targets:
            if self.exit:
                break

            # 1処理を行う前に実行するコールバックを
            # 呼び出す
            if self.statusView:
                self.statusView.set_status_text(src)
                
            # src, dstに対して所定の処理を実行
            self.execute(src, dst)

            # １処理完了直後にステータスビューを更新
            if self.statusView:
                self.statusView.increment()

            time.sleep(0.01)

        # ステータスバーの表示をクリアする
        if self.statusView:
            self.statusView.set_fraction(0)
            self.statusView.clear_progress_text()

            msg = 'SUCCEEDED'
            if self.errorCnt > 0:
                msg = '%d ERRORS' % self.errorCnt
            self.statusView.set_status_text(msg)

        # 完了通知用コールバックの呼び出し
        if self.cbFinished:
            self.cbFinished()

    def execute(self, src, dst):
        '''
        処理元(src)に何らかの処理を行い、出力先(dst)に
        コピーする。
        '''
        if src:
            src = src.decode('utf-8')
        if dst:
            dst = dst.decode('utf-8')

        # コピー先が指定されていなかったらスキップ
        if dst == None or len(dst) == 0:
            return

        dstDName, dstFName = os.path.split(dst)
        # ファイル名の長さが0だったらスキップ
        if len(dstFName) == 0:
            return

        # コピー先のディレクトリが存在しなかったら
        # 新規に作成
        if os.path.isdir(dstDName) == False:
            os.makedir(dstDName)

        # コピーの実行
        try:
            print '[SRC] %s' % src
            self.copyFunc(src, dst)
            print '-->[DST] %s' % dst
        except IOError, err:
            self.errorCnt += 1
            print '-->IOError(%s) : %s' % (err.errno,
                                           err.strerror)
            # コールバック関数の呼び出し
            if self.cbError:
                self.cbError(err.errno, err.strerror, src, dst)

    def stop(self):
        '''
        スレッド停止用フラグを設定する。
        '''
        self.exit = True
