# -*- coding: utf-8 -*-

import sys
import os
import shutil
import xml.sax

import pygtk
if sys.platform == 'win32':
    pygtk.require('2.0')
import gtk

#
# PYTHONPATHに独自モジュールへのパスを追加
gModulePathList = ['dialogs']
for modPath in gModulePathList:
    sys.path.append(modPath)

from resource import *

from fcpExceptions import *
from fcpDoc import *
from fcpPreference import *
from fcpFileTreeModel import *
from fcpRunThread import *
from statusView import StatusView
from logMgr import FCPLogMgr

from dialogs.aboutDialog import AboutDialog
from dialogs.fileSelectDialog import FileSelectDialog
from dialogs.keywordSelectDialog import KeywordSelectDialog
from dialogs.simpleDialogs import SingleEntryDialog
from dialogs.indexingDialog import IndexingDialog
from dialogs.preferenceDialog import FCPPreferenceDialog
from dialogs.replacePathDialog import ReplacePathDialog

# GTKスレッドの初期化
gtk.gdk.threads_init()



#
#
class FCPEditWindow:
    '''
    pyFCPウインドウクラス。
    '''
    def __init__(self):
        '''
        初期化
        '''        
        self.savedPathUtf8 = None
        self.modified =False
        self.runThread = None

        # ツールオプション保持用のオブジェクトを生成
        self.toolPref = self.get_tool_preference()

        # ログマネージャを生成
        self.logMgr = FCPLogMgr()

        # ファイルダイアログのデフォルトディレクトリを
        # プリファレンスから読み込む
        self.fileChooserDir = self.toolPref.get_value(PREF_KEY_DEFAULT_DIRECTORY)
        
        # データモデルの生成
        self.fcpModel = FCPModel()

        # データ表示用モデルの作成
        self.dispModel = FCPDispModel()

        # ツリービューの作成
        enableTreeLine = self.toolPref.get_value(PREF_KEY_VIEW_DRAW_TREE_LINE)
        model = self.fcpModel.get_model()
        self.treeView = self.dispModel.construct_view(model,
                                                      enableTreeLine)
        sel = self.treeView.get_selection()
        if sel:
            sel.set_mode(gtk.SELECTION_MULTIPLE)

        # 編集ウインドウの作成
        self.wind = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.wind.set_title('untitled')
        
        windW, windH = self.toolPref.get_value(PREF_KEY_WINDOW_SIZE)
        self.wind.set_default_size(windW, windH)
        
        windX, windY = self.toolPref.get_value(PREF_KEY_WINDOW_POS)
        self.wind.move(windX, windY)
        
        self.wind.connect('destroy', self.on_exit)
        self.wind.connect('delete_event', self.on_delete_event)

        # メニューバーの作成
        self.menubar = self.get_main_menubar()
        
        # ツリービューをウインドウ上に配置
        vbox = gtk.VBox()
        vbox.pack_start(self.menubar, False, False, 4)
        
        scrl = gtk.ScrolledWindow()
        scrl.add(self.treeView)
        vbox.pack_start(scrl)

        # ステータスバーを配置
        self.statusView = StatusView()
        vbox.pack_start(self.statusView, False, False, 4)
        
        self.wind.add(vbox)  

        # ウインドウの表示
        self.wind.show_all()

    def get_tool_preference(self):
        '''
        ツール設定オブジェクトの生成
        '''
        pref = FCPPreference()

        if os.path.isfile(FCP_RES_STR_PATH_PREFERENCE) == False:
            return pref
        
        # XMLファイルを読み込む
        try:
            f = file(FCP_RES_STR_PATH_PREFERENCE, 'r')
            xmlText = f.read()
            f.close()
        except IOError, err:
            fcpExceptions.show_exception_message(err.errno,
                                                 err.message,
                                                 err.strerror)
            f.close()
            return None
        
        # XML中のツール設定をプリファレンスへ反映
        pref.set_xml_text(xmlText)
        return pref

    def save_tool_preference(self):
        '''
        ツール設定を保存する。
        '''
        if self.toolPref == None:
            return

        # 直前にファイルダイアログで移動したディレクトリを
        # デフォルトディレクトリに設定する。
        if self.toolPref.get_value(PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT):
            self.toolPref.set_value(PREF_KEY_DEFAULT_DIRECTORY,
                                    self.fileChooserDir)

        # ウインドウ情報（デフォルトの位置、サイズ）
        if self.wind:
            # 位置
            if self.toolPref.get_value(PREF_KEY_SAVE_WINDW_POS):
                self.toolPref.set_value(PREF_KEY_WINDOW_POS,
                                        self.wind.get_position())
            # サイズ
            if self.toolPref.get_value(PREF_KEY_SAVE_WINDW_SIZE):
                self.toolPref.set_value(PREF_KEY_WINDOW_SIZE,
                                        self.wind.get_size())

        # プリファレンスの設定内容をXMLテキストに。
        xmlText = self.toolPref.get_xml_text()
        if len(xmlText) == 0:
            return

        # ファイルへの保存
        try:
            f = file(FCP_RES_STR_PATH_PREFERENCE, 'w+')
            f.write(xmlText)
            f.close()
        except IOError, err:
            fcpExceptions.show_exception_message(err.errno,
                                                 err.message,
                                                 err.strerror)
            f.close()

    def get_main_menubar(self):
        '''
        メニューバーの構築
        '''
        # UIManagerの生成
        self.menuMgr = gtk.UIManager()

        # アクセラレータグループをウインドウに追加
        acclGrp = self.menuMgr.get_accel_group()
        self.wind.add_accel_group(acclGrp)

        # アクショングループの作成
        actGrpBase = gtk.ActionGroup('MenuActionGrpBase')
        actGrpBase.add_actions(
            [('File', None, '_File'),
             ('Edit', None, '_Edit'),
             ('Run', None, '_Run'),
             ('Selection', None, '_Selection'),
             ('View', None, '_View'),
             ('Option', None, '_Option'),
             ('Directory settings', None, '_Directory settings'),
             ('Window settings', None, '_Window settings'),
             ('View settings', None, '_View settings'),
             ('Log settings', None, '_Log settings'),
             ('Help', None, '_Help')])
        self.menuMgr.insert_action_group(actGrpBase, 0)

        # Fileメニューアクションの登録        
        actGrpBase.add_actions(
            [('New...', None,
              '_New', '<control>n',
              'Create new setting',
              self.on_menu_file_new),

             ('Open...', None,
              '_Open...', '<control>o',
              'Open setting file(.xml)',
              self.on_menu_file_open),

             ('Save...', None,
              '_Save...', '<control>s',
              'Save setting file(.xml)',
              self.on_menu_file_save),

             ('Save as...', None,
              'Save as...', None,
              'Save setting file(.xml) with another name',
              self.on_menu_file_save_as),

             ('Exit', None,
              'E_xit', None,
              'Exit this tool',
              self.on_exit)])

        # Editメニューアクションの生成
        actGrpBase.add_actions(
            [('Choose source files...', None,
              'Choose _source files...', None,
              'Choose source files to copy.',
              self.on_menu_edit_choose_source),

             ('Remove source files', None,
              'Remove source files', None,
              'Remove seelcted sources files.',
              self.on_menu_edit_remove_source),
                                
             ('Choose destination directory...', None,
              'Choose _destination directory...', None,
              'Set destination directory of sources.',
              self.on_menu_edit_choose_destination),

             ('Indexing destination files...', None,
              '_Indexing destination files...', None,
              'Indexing all destination file names.',
              self.on_menu_edit_indexed_destination),

             ('Clear destination directory...', None,
              '_Clear destination directory...', None,
              'Clear destination directory of sources',
              self.on_menu_edit_clear_destination),

             ('Swap srouce/destination path', None,
              'S_wap source/destination path', None,
              'Swap all source path and destination path.',
              self.on_menu_edit_swap_src_dst),

             ('Replace path string...', None,
              '_Replace path string...', None,
              'Replace path string.',
              self.on_menu_edit_replace_path)])

        # Runメニューアクションの生成
        actGrpBase.add_actions(
            [('Start', None,
              '_Start', None,
              'Start to run.',
              self.on_menu_run_start),

             ('Cancel', None,
              '_Cancel', None,
              'Cancel to run.',
              self.on_menu_run_cancel)])
        
        # Selectメニューアクションの生成
        actGrpBase.add_actions(
            [('Source files which include extension...', None,
              'Source files which _include extension...', None,
              'Select source files include extionsion.',
              self.on_menu_select_source_include_ext),

             ('Source files which does not include extension...', None,
              'Source files which does not include _extension...', None,
              'Select source files which does not include extionsion.',
              self.on_menu_select_source_exclude_ext),

             ('Select all', None,
              'Select _all', None,
              'Select all rows.',
              self.on_menu_select_select_all),
                                
             ('Toggle selection', None,
              '_Toggle selection', None,
              'Toggle selection.',
              self.on_menu_select_toggle_selection),
                                
             ('Unselect all', None,
              'U_nselect all', None,
              'Unselect all rows.',
              self.on_menu_select_unselect_all)])

        # Viewメニューアクションのn生成
        actGrpBase.add_actions(
            [('Expand all', None,
              '_Expand all', None,
              'Expand all rows',
              self.on_menu_view_expand_all),
                                
             ('Collapse all', None,
              '_Collapse all', None,
              'Collapse all rows',
              self.on_menu_view_collapse_all)])
        
        # Optionメニューアクションの生成
        actGrpBase.add_toggle_actions(
            [('Keep file dialog directory', None,
              'Keep file dialog _directory', None,
              'Keep latest file dialog directory',
              self.on_menu_option_keep_file_dialog_dir,
              self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY)),
                                       
             ('Save file dialog directory as default', None,
              'Save file dialog directory _as default', None,
              'When quitting, save file dialog directory as the default directory.',
              self.on_menu_option_save_file_dialog_as_default,
              self.toolPref.get_value(PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT)),
                                       
             ('Save window position', None,
              'Save window _position', None,
              'Save window position.',
              self.on_menu_option_save_window_position,
              self.toolPref.get_value(PREF_KEY_SAVE_WINDW_POS)),

             ('Save window size', None,
              'Save window _size', None,
              'Save window size.',
              self.on_menu_option_save_window_size,
              self.toolPref.get_value(PREF_KEY_SAVE_WINDW_SIZE)),

             ('Always expand source when it appended', None,
              'Always _expand source when it appended', None,
              'Always expand source when it appended',
              self.on_menu_option_always_expand_tree,
              self.toolPref.get_value(PREF_KEY_VIEW_TREE_EXPANDED)),

             ('Select appended rows', None,
              'Select appended _rows', None,
              'Select rows when source files appended.',
              self.on_menu_option_select_src_rows,
              self.toolPref.get_value(PREF_KEY_VIEW_SELECT_APPENDED_ROWS)),
             
             ('Draw tree lines', None,
              'Draw tree _lines', None,
              'Draw tree lines',
              self.on_menu_option_draw_tree_lines,
              self.toolPref.get_value(PREF_KEY_VIEW_DRAW_TREE_LINE)),

             ('Show toolbar', None,
              'Show _toolbar', None,
              'Show toolbar',
              self.on_menu_option_show_toolbar,
              self.toolPref.get_value(PREF_KEY_VIEW_SHOW_TOOLBAR)),

             ('Export error log file', None,
              'Export _error log file', None,
              'Export log file when error occur.',
              self.on_menu_option_log_export,
              self.toolPref.get_value(PREF_KEY_LOG_EXPORT)),

             ('Export error log file in append mode', None,
              'Export error log file in _append mode', None,
              'Export error log file in append mode',
              self.on_menu_optioin_log_append_mode,
              self.toolPref.get_value(PREF_KEY_LOG_APPEND_MODE))])

        actGrpBase.add_actions(
            [('Save option...', None,
              'Save _option...', None,
              'Save current option',
              self.on_menu_option_save_option),
                                
             ('Other settings...', None,
              'Oth_er settings...', None,
              'Open setting dialog',
              self.on_menu_option_other_settings)])

        # Helpメニューアクションの生成
        actGrpBase.add_actions(
            [('About this tool...', None,
              'About this tool...', None,
              'Show tool info dialog',
              self.on_menu_help_about)])

        self.menuMgr.add_ui_from_string(FCP_MAIN_MENU)
        return self.menuMgr.get_widget('/MenuBar')

    def reset(self):
        '''
        ツールウインドウの内容を起動直後の状態に戻す。
        '''
        print '--- RESET ---'
        # 未保存の場合には保存確認、および保存を行う。
        if self.get_modified():
            if self.show_confirm(FCP_RES_STR_CONFIRM_SAVE_BF_NEW,
                                 gtk.MESSAGE_QUESTION,
                                 gtk.BUTTONS_YES_NO):

                self.on_menu_file_save(widget, event, data)

        # ツリーモデルのデータをクリア
        self.fcpModel.clear_targets()

        # ステータスバーの表示をクリア
        self.statusView.set_status_text('')
        
        # そのほかの状態をクリア
        self.savedPathUtf8 = None
        self.modified =False
        self.runThread = None

        # ウインドウタイトルをuntitledに
        self.wind.set_title('untitled')

    def set_modified(self, flag):
        '''
        編集フラグを設定する
        '''
        self.modified = flag

    def get_modified(self):
        '''
        編集フラグを取得する
        '''
        return self.modified
    
    def select_row_by_extension(self, match=True):
        '''
        拡張子を指定してファイルを選択する。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        # リスト選択用の拡張子をプリファレンスから取得
        extList = self.toolPref.get_value(PREF_KEY_LIST_FILTER_EXTENSIONS)
        dlg = KeywordSelectDialog(title=FCP_RES_STR_EXTENSION_SELECT_TITLE,
                                  keywords=extList)

        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            # ダイアログで設定された拡張子を取得し、拡張子が一致する
            # ツリービューのRowのイテレータを取得。
            ext = dlg.get_active_keyword()
            print ext
            iterList = self.fcpModel.iter_files_by_extension(ext, match)

            # 取得したイテレータでツリービューの行を選択状態
            # にする。
            sel = self.treeView.get_selection()
            if sel:
                sel.unselect_all()
                for aIter in iterList:
                    sel.select_iter(aIter)

        dlg.destroy()        
        return True

    def get_expanded_rows(self):
        '''
        展開状態になっているRowをリストとして返す
        '''
        rows = []
        if self.treeView == None or self.fcpModel == None:
            return rows

        # データモデルを取得する
        model = self.fcpModel.get_model()
        if model == None:
            return rows
        
        # データモデルのトップレベルのRowのイテレータを取得
        aIter = model.get_iter_first()
        if aIter == None:
            return

        # トップレベルのRow展開(expand)状態を調べ
        # 展開されているものだけのリストを作成する
        while aIter != None:
            rowPath = model.get_path(aIter)
            if self.treeView.row_expanded(rowPath):
                rows.append(rowPath)
            aIter = model.iter_next(aIter)

        return rows

    def show_confirm(self, msg,
                     msgType=gtk.MESSAGE_QUESTION,
                     msgButtons=gtk.BUTTONS_YES_NO):
        '''
        未保存時の保存確認ダイアログを表示する。
        '''
        # ダイアログを表示
        msgDlg = gtk.MessageDialog(parent=None,
                                   flags=gtk.DIALOG_MODAL,
                                   type=msgType,
                                   buttons=msgButtons,
                                   message_format=msg)
        msgDlg.set_position(gtk.WIN_POS_CENTER)
        #msgDlg.set_size_request(300, -1)

        response = msgDlg.run()
        msgDlg.destroy()

        return response == gtk.RESPONSE_YES

    def select_src_rows(self, srcList):
        '''
        srcListで渡されたパスに該当するコピー元行を選択する。
        '''
        if self.fcpModel == None or self.treeView == None:
            return        
        if srcList == None:
            return
        
        targetIters = self.fcpModel.get_src_iters(srcList)
        selection = self.treeView.get_selection()
        selection.unselect_all()

        if targetIters:
            for aIter in targetIters:
                selection.select_iter(aIter)
        
    def on_delete_event(self, widget, event=None, data=None):
        '''
        クローズボックスでウインドウが閉じられた場合に
        実行素べき処理。
        '''
        self.on_exit(widget, event, data)
        return True
    
    def on_menu_file_new(self, widget, event=None, data=None):
        '''
        別ウインドウを表示するのではなく、ウインドウの内容をクリアする。
        '''
        self.reset()

    def on_menu_file_open(self, widget, event=None, data=None):
        '''
        XMLファイルを読み込んでTreeViewに設定する。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        dlg = FileSelectDialog(title='Choose xml file name to open',
                               action=gtk.FILE_CHOOSER_ACTION_SAVE)

        dlg.set_current_folder(self.fileChooserDir)
        
        filterInfo = [('XML file', '*.xml')]
        dlg.set_filter_info(filterInfo)

        response = dlg.run()

        # ファイルダイアログのディレクトリを保持する
        # 設定になっている場合には、ディレクトリ情報を更新
        if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
            self.fileChooserDir = dlg.get_current_folder()
        
        if response == gtk.RESPONSE_OK:
            # パスの文字コードをUTF8に変換
            path = dlg.get_filenames()[0]
            utf8Path = path.decode('utf-8')
            
            # XMLファイルを読み込む
            try:
                f = file(os.path.normpath(utf8Path), 'r')
                text = f.read()
                f.close()
            except IOError, err:
                fcpExceptions.show_exception_message(err.errno,
                                                     err.message,
                                                     err.strerror)
                dlg.destroy()
                f.close()
                return
            
            # XMLテキストをパース
            parser = xml.sax.make_parser()            
            handler = FCPDocHandler()
            parser.setContentHandler(handler)
            try:
                parser.feed(text)
            except SAXParseException, err:
                fcpExceptions.show_exception_message(err.errno,
                                                     err.message,
                                                     err.strerror)
                return

            # パースしたパス情報をツリーモデルに設定
            targetList = handler.get_targets()
            self.fcpModel.set_targets(targetList)

            # オプションのEXPAND ALLが選択されている場合には
            # ツリーを展開する。            
            if self.toolPref.get_value(PREF_KEY_VIEW_TREE_EXPANDED):
                self.treeView.expand_all()

            # 今回登録した行を選択状態にする。
            if self.toolPref.get_value(PREF_KEY_VIEW_SELECT_APPENDED_ROWS):
                self.select_src_rows([s for s, d in targetList])
            
            # パスを保持し、ウインドウタイトルとして設定
            self.savedPathUtf8 = utf8Path
            self.wind.set_title(self.savedPathUtf8)
            
        dlg.destroy()
        
    def save_to_xml(self, path):
        '''
        指定されたパスへXMLファイルを保存する。
        pathはUTF8に変換される。
        '''
        targets = self.fcpModel.get_targets()

        # XMLを生成
        doc = FCPDocGenerator()
        xmlText = doc.generate(targets)

        # XMLをファイルに保存
        name, ext = os.path.splitext(path)
        if len(ext) == 0 or ext != '.xml':
            path += '.xml'

        utf8Path = path.decode('utf-8')        
        try:
            f = file(os.path.normpath(utf8Path), 'w')
            f.write(xmlText)
            f.close()
        except IOError, err:
            fcpExceptions.show_exception_message(err.errno,
                                                 err.message,
                                                 err.strerror)
            f.close()
            return

        # 保存したファイルパスを退避し、ウインドウタイトルも設定
        self.savedPathUtf8 = utf8Path
        self.wind.set_title(self.savedPathUtf8)
        
    def on_menu_file_save(self, widget, event=None, data=None):
        '''
        TreeViewに登録されている内容をXMLファイルとして保存する。
        '''
        if self.fcpModel == None:
            return True

        path = self.savedPathUtf8
        if path == None or os.path.isfile(path) == False:
            # 一度も保存していない場合、もしくは、保存してあった
            # ファイルが存在しなくなっている場合にはファイル選択
            # ダイアログを表示させる
            dlg = FileSelectDialog(title='Choose xml file name to save',
                                   action=gtk.FILE_CHOOSER_ACTION_SAVE)

            dlg.set_current_folder(self.fileChooserDir)

            filterInfo = [('XML file', '*.xml')]
            dlg.set_filter_info(filterInfo)
            dlg.set_overwrite_confirmation(True)

            response = dlg.run()

            # ファイルダイアログのディレクトリを保持する
            # 設定になっている場合には、ディレクトリ情報を更新
            if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
                self.fileChooserDir = dlg.get_current_folder()

            if response != gtk.RESPONSE_OK:
                dlg.destroy()
                return True
            
            path = dlg.get_filenames()[0]
            dlg.destroy()
                        
        self.save_to_xml(path)

    def on_menu_file_save_as(self, widget, event=None, data=None):
        '''
        TreeViewに登録されている内容を別名のXMLファイルとして保存する。
        '''
        if self.fcpModel == None:
            return True

        dlg = FileSelectDialog(title='Choose xml file name to save',
                               action=gtk.FILE_CHOOSER_ACTION_SAVE)

        dlg.set_current_folder(self.fileChooserDir)

        filterInfo = [('XML file', '*.xml')]
        dlg.set_filter_info(filterInfo)
        dlg.set_overwrite_confirmation(True)

        response = dlg.run()

        # ファイルダイアログのディレクトリを保持する
        # 設定になっている場合には、ディレクトリ情報を更新
        if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
            self.fileChooserDir = dlg.get_current_folder()

        if response == gtk.RESPONSE_OK:
            path = dlg.get_filenames()[0]                        
            self.save_to_xml(path)

        dlg.destroy()
        return True

    def on_exit(self, widget, event=None, data=None):
        '''
        終了時の処理。
        GTKのイベントループを終了させる。
        '''
        print '--- quit ---'

        # 未保存の場合には保存確認、および保存を行う。
        if self.get_modified():
            if self.show_confirm(FCP_RES_STR_CONFIRM_SAVE_BF_EXIT,
                                 gtk.MESSAGE_QUESTION,
                                 gtk.BUTTONS_YES_NO):

                self.on_menu_file_save(widget, event, data)
        # ツール設定を出力する。
        self.save_tool_preference()

        # GTKの終了
        gtk.main_quit()
        
    def on_menu_edit_choose_source(self, widget, event=None, data=None):
        '''
        ソース選択処理。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True
        
        dlg = FileSelectDialog('Choose files for source')
        dlg.set_select_multiple(True)
        dlg.set_current_folder(self.fileChooserDir)

        # ファイルダイアログのフィルタをぷリファレンスから取得
        filters = self.toolPref.get_value(PREF_KEY_FILE_FILTER_EXTENSIONS)
        dlg.set_filter_info(filters)
        
        response = dlg.run()

        # ファイルダイアログのディレクトリを保持する
        # 設定になっている場合には、ディレクトリ情報を更新
        if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
            self.fileChooserDir = dlg.get_current_folder()

        if response == gtk.RESPONSE_OK:
            files = dlg.get_filenames()
            targets = [(f, None) for f in files]
            self.fcpModel.append_targets(targets)

            # オプションのEXPAND ALLが選択されている場合には
            # ツリーを展開する。            
            if self.toolPref.get_value(PREF_KEY_VIEW_TREE_EXPANDED):
                self.treeView.expand_all()

            # 追加されたコピー元行を選択状態にする。
            if self.toolPref.get_value(PREF_KEY_VIEW_SELECT_APPENDED_ROWS):
                self.select_src_rows(files)

        dlg.destroy()
        self.set_modified(True)
        return True

    def on_menu_edit_remove_source(self, widget, event=None, data=None):
        '''
        現在選択されている行を削除する。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        sel = self.treeView.get_selection()
        if sel == None:
            return True

        model, rows = sel.get_selected_rows()
        self.fcpModel.remove_targets(rows)
        self.set_modified(True)
        return True

    def on_menu_edit_replace_path(self, widget, event=None, data=None):
        '''
        コピー元のファイルパスを置き換える
        '''
        if self.fcpModel == None:
            return True
        
        dlg = ReplacePathDialog(self.wind)
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            searchText = dlg.get_text_for_search()
            replaceText = dlg.get_text_for_replace()
            targetStr = dlg.get_target()
                        
            # 置き換え処理
            self.fcpModel.replace_string(searchText,
                                         replaceText,
                                         targetStr)
        dlg.destroy()

    def on_menu_edit_choose_destination(self, widget, event=None, data=None):
        '''
        現在選択されているコピー元元に対してコピー先を指定する。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        dlg = FileSelectDialog(title='Choose files for source',
                               action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        dlg.set_current_folder(self.fileChooserDir)

        response = dlg.run()

        # ファイルダイアログのディレクトリを保持する
        # 設定になっている場合には、ディレクトリ情報を更新
        if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
            self.fileChooserDir = dlg.get_current_folder()

        if response == gtk.RESPONSE_OK:
            sel = self.treeView.get_selection()
            if sel == None:
                return True

            model, rows = sel.get_selected_rows()
            self.fcpModel.set_destination(dlg.get_filenames()[0], rows)

        dlg.destroy()
        self.set_modified(True)
        return True

    def on_menu_edit_indexed_destination(self, widget, event=None, data=None):
        '''
        行の選択状態にかかわらず、全てのファイルの出力結果名に
        連番のファイルを付ける。
        '''
        if self.fcpModel == None:
            return True
        
        dlg = IndexingDialog()
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            form = dlg.get_text()
            keepExt = dlg.get_keep_extention()
            self.fcpModel.set_indexed_destination(0, form, keepExt)

        dlg.destroy()
        self.set_modified(True)
        return True

    def on_menu_edit_clear_destination(self, widget, event=None, data=None):
        '''
        選択されている行のコピー先をクリアする。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        sel = self.treeView.get_selection()
        if sel == None:
            return True

        # 本当に削除するか確認。
        if self.show_confirm(FCP_RES_STR_CONFIRM_CLEAR_DEST) == False:
            return
        
        model, rows = sel.get_selected_rows()
        self.fcpModel.set_destination(None, rows)
        self.set_modified(True)
        return True

    def on_menu_edit_swap_src_dst(self, widget, event=None, data=None):
        '''
        コピー元パスとコピー先パスを入れ替える。
        '''
        if self.fcpModel == None or self.treeView == None:
            return True

        # 処理対象パスと出力先パスを入れ替える
        targets = self.fcpModel.get_targets()
        swList = map(lambda a: (a[1], a[0]), targets)

        expandedRows = self.get_expanded_rows()
        
        # 全てのターゲットを削除し、追加しなおす。
        self.fcpModel.clear_targets()
        self.fcpModel.append_targets(swList)

        for row in expandedRows:
            self.treeView.expand_row(row, False)

        self.set_modified(True)
        return True

    def cb_run_thread_error(self, errno, errstr, src, dst):
        '''
        処理スレッドでエラーが発生するたびに呼び出されるコールバック
        '''
        if self.logMgr == None:
            return

        log = (errno, errstr.decode('utf-8'), src, dst)
        self.logMgr.push(log)

    def cb_run_thread_finished(self):
        '''
        処理スレッド完了時に呼び出されるコールバック
        '''        
        print '-- finished --'
        if self.toolPref == None or self.logMgr == None:
            return

        if self.toolPref.get_value(PREF_KEY_LOG_EXPORT) == False:
            return
        
        logDir = self.toolPref.get_value(PREF_KEY_LOG_DIRECTORY)
        appendMode = self.toolPref.get_value(PREF_KEY_LOG_APPEND_MODE)

        # 実行ログの書きだし
        self.logMgr.save(os.path.join(logDir, FCP_RES_STR_LOG_NAME),
                         {True: 'a+', False: 'w+'}[appendMode])
    
    def on_menu_run_start(self, widget, event=None, data=None):
        '''
        処理を実行する。
        '''
        if self.fcpModel == None:
            return True

        # ログを一旦クリア
        if self.logMgr:
            self.logMgr.clear()

        self.runThread = FCPRunThread(self.fcpModel.get_targets(),
                                      self.statusView,
                                      self.cb_run_thread_error,
                                      self.cb_run_thread_finished)

        # プリファレンスの設定に応じてコピー関数を切り替える
        flag = self.toolPref.get_value(PREF_KEY_FILE_KEEP_SOURCE_DATE)
        funcDict = {True: shutil.copy2, False: shutil.copy}
        self.runThread.set_copy_func(funcDict[flag])

        # 処理スレッドを開始
        self.runThread.start()
        return True

    def on_menu_run_cancel(self, widget, event=None, data=None):
        '''
        処理を中止する。
        '''
        if self.runThread:
            self.runThread.stop()
            self.runThread = None
        return True

    def on_menu_select_source_include_ext(self, widget, event=None, data=None):
        '''
        拡張子が一致する行を選択する。
        '''
        self.select_row_by_extension()
        return True
    
    def on_menu_select_source_exclude_ext(self, widget, event=None, data=None):
        '''
        拡張子が一致しない行を選択する。
        '''
        self.select_row_by_extension(False)
        return True

    def on_menu_select_select_all(self, widget, event=None, data=None):
        '''
        全ての行の選択状態にする。
        '''
        if self.treeView == None:
            return True
        
        sel = self.treeView.get_selection()
        if sel:
            sel.select_all()
        return True

    def toggle_selection_func(self, model, path, aIter, selection):
        '''
        TreeViewの選択状態を反転させるための関数。
        リターン値は常にFalse。
        '''
        # トップレベルの行（ディレクトリの行）はトグルしない。
        if model.iter_has_child(aIter):
            selection.unselect_iter(aIter)
            return False
        
        if selection == None or aIter == None:
            return False
        if selection.iter_is_selected(aIter):
            selection.unselect_iter(aIter)
        else:
            selection.select_iter(aIter)
        return False

    def on_menu_select_toggle_selection(self, widget, event=None, data=None):
        '''
        選択状態を反転させる。
        '''
        if self.treeView == None or self.fcpModel == None:
            return True

        model = self.fcpModel.get_model()
        if model == None:
            return
        
        sel = self.treeView.get_selection()
        if sel:
            model.foreach(self.toggle_selection_func, sel)

        return True
            
    def on_menu_select_unselect_all(self, widget, event=None, data=None):
        '''
        全ての行の選択状態を解除する。
        '''
        if self.treeView == None:
            return True
        
        sel = self.treeView.get_selection()
        if sel:
            sel.unselect_all()
        return True

    def on_menu_view_expand_all(self, widget, event=None, data=None):
        '''
        全ての行のツリー表示を展開状態にする。
        '''
        if self.treeView == None:
            return
        self.treeView.expand_all()
        return True
    
    def on_menu_view_collapse_all(self, widget, event=None, data=None):
        '''
        全ての行のツリー表示を非展開状態にする。
        '''
        if self.treeView == None:
            return
        self.treeView.collapse_all()
        return True

    def on_menu_option_keep_file_dialog_dir(self, widget, event=None, data=None):
        '''
        ファイル選択ダイアログのディレクトリを保持するかどうかの
        フラグをツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_KEEP_FILE_DIRECTORY,
                                widget.get_active())
        return True

    def on_menu_option_save_file_dialog_as_default(self, widget, event=None, data=None):
        '''
        ツール終了時にファイル選択ダイアログのディレクトリをデフォルトディレクトリ
        として反映させるかどうかのフラグをツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_SAVE_FILE_DIR_AS_DEFAULT,
                                widget.get_active())
        return True
        
    def on_menu_option_save_window_position(self, widget, event=None, data=None):
        '''
        ウインドウの座標を保存するかどうかのフラグを
        ツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_SAVE_WINDW_POS,
                                widget.get_active())
        return True
    
    def on_menu_option_save_window_size(self, widget, event=None, data=None):
        '''
        ウインドウのサイズを保存するかどうかのフラグを
        ツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_SAVE_WINDW_SIZE,
                                widget.get_active())
        return True
    
    def on_menu_option_always_expand_tree(self, widget, event=None, data=None):
        '''
        コピー元ディレクトリが追加された場合に、追加された行を展開状態にするか
        どうかのフラグをツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_VIEW_TREE_EXPANDED,
                                widget.get_active())
        return True

    def on_menu_option_select_src_rows(self, widget, event=None, data=None):
        '''
        コピー元ファイルが追加された時に、追加された行を選択状態にするか
        どうかのフラグをツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_VIEW_SELECT_APPENDED_ROWS,
                                widget.get_active())
        return True
        
    def on_menu_option_draw_tree_lines(self, widget, event=None, data=None):
        '''
        ツリーにラインを表示するかどうかのフラグを
        ツールオプションに設定。
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return

        enable = widget.get_active()
        self.toolPref.set_value(PREF_KEY_VIEW_DRAW_TREE_LINE, enable)
        
        if self.treeView:
            self.treeView.set_enable_tree_lines(enable)
        return True
    
    def on_menu_option_show_toolbar(self, widget, event=None, data=None):
        '''
        ツールバーの表示をトグルさせるかどうか
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_VIEW_SHOW_TOOLBAR,
                                widget.get_active())
        return True

    def on_menu_option_log_export(self, widget, event=None, data=None):
        '''
        エラーログを出力するかどうか
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_LOG_EXPORT,
                                widget.get_active())
        return True

    def on_menu_optioin_log_append_mode(self, widget, event=None, data=None):
        '''
        ログを追記モードで出力するかどうか
        '''
        if self.toolPref == None:
            return
        if widget == None:
            return
        self.toolPref.set_value(PREF_KEY_LOG_APPEND_MODE,
                                widget.get_active())
        return True
    
    def on_menu_option_save_option(self, widget, event=None, data=None):
        '''
        オプションを保存する。
        '''
        self.save_tool_preference()
        return True
    
    def on_menu_option_other_settings(self, widget, event=None, data=None):
        '''
        設定ダイアログを表示
        '''
        fileFilters = self.toolPref.get_value(PREF_KEY_FILE_FILTER_EXTENSIONS)
        listFilters = self.toolPref.get_value(PREF_KEY_LIST_FILTER_EXTENSIONS)
        dlg = FCPPreferenceDialog(self.wind,
                                  fileFilters,
                                  listFilters)
        dlg.set_current_folder(self.fileChooserDir)
        
        # ファイルダイアログのデフォルトディレクトリを反映
        path = self.toolPref.get_value(PREF_KEY_DEFAULT_DIRECTORY)
        dlg.set_default_dir(path)
        
        # ログファイルの出力ディレクトリを反映
        path = self.toolPref.get_value(PREF_KEY_LOG_DIRECTORY)
        dlg.set_log_dir(path)

        # コピーしたファイルの日付はコピー元の日付にするかどうかの
        # 設定を反映
        flag = self.toolPref.get_value(PREF_KEY_FILE_KEEP_SOURCE_DATE)
        dlg.set_keep_file_date(flag)
        
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            # ファイルダイアログのデフォルトディレクトリを反映
            self.toolPref.set_value(PREF_KEY_DEFAULT_DIRECTORY,
                                    dlg.get_default_dir())

            # ログファイルの出力ディレクトリを反映
            self.toolPref.set_value(PREF_KEY_LOG_DIRECTORY,
                                    dlg.get_log_dir())

            # ファイルフィルタを反映
            self.toolPref.set_value(PREF_KEY_FILE_FILTER_EXTENSIONS,
                                    dlg.get_file_filters())
            
            # リストフィルタを反映
            self.toolPref.set_value(PREF_KEY_LIST_FILTER_EXTENSIONS,
                                    dlg.get_list_filters())
            
            # コピーしたファイルの日付はコピー元の日付にするかどうかの
            # 設定を反映
            self.toolPref.set_value(PREF_KEY_FILE_KEEP_SOURCE_DATE,
                                    dlg.get_keep_file_date())

        # ファイルダイアログのディレクトリを保持する
        # 設定になっている場合には、ディレクトリ情報を更新
        if self.toolPref.get_value(PREF_KEY_KEEP_FILE_DIRECTORY):
            self.fileChooserDir = dlg.get_current_folder()

        dlg.destroy()

    def on_menu_help_about(self, widget, event=None, data=None):
        '''
        Aboutダイアログを表示
        '''
        dlg = AboutDialog()
        response = dlg.run()
        dlg.destroy()

#
#
def main():
    # パッケージファイル編集ウインドウ表示
    wind = FCPEditWindow()

    # GTKのイベントループ開始
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()

#
#
if __name__ == '__main__':
    main()
